<?php

/**
 * Класс управления событиями
 * @doc https://docs.instantcms.ru/dev/controllers/hooks
 */
class cmsEventsManager {

    /**
     * Список всех слушателей и событий
     * @var array
     */
    private static $structure = null;

    /**
     * Оповещает слушателей о произошедшем событии
     * Входящие данные $data передаются каждому слушателю по очереди,
     * на выходе возвращается измененный слушателями параметр $data
     *
     * @param mixed $event_name Название события/массив событий
     * @param mixed $data Параметр события
     * @param mixed $default_return Значение, возвращаемое по умолчанию если у события нет слушателей
     * @param object $_request Объект запроса
     * @return array Обработанный массив данных
     */
    public static function hook($event_name, $data = false, $default_return = null, $_request = false) {

        // Используйте массив событий, если они с разным названиями,
        // но с одинаковыми параметрами
        if (is_array($event_name)) {

            foreach ($event_name as $_event_name) {
                $data = self::hook($_event_name, $data, $default_return, $_request);
            }

            return $data;
        }

        // >> @WebMan - debugx
        $dbg_event_data = $data;

        cmsDebugging::pointStart('x_events');
        // << @WebMan - debugx

        //получаем все активные контроллеры, привязанные к указанному событию
        $listeners = self::getEventListeners($event_name);

        //если активных контроллеров нет, возвращаем данные без изменений
        if (!$listeners) {

            // >> @WebMan - debugx
            cmsDebugging::pointProcess('events_empty', function () use($event_name, $data) {
                return [
                    'data'     => 'hook => '.$event_name,
                    'x_name'   => $event_name,
                    'x_action' => 'hook',
                    'x_data'   => $data
                ];
            }, 1);
            // << @WebMan - debugx

            return is_null($default_return) ? $data : $default_return;
        }

        //перебираем контроллеры и вызываем каждый из них, передавая $data
        foreach ($listeners as $listener) {

            $request = ($_request === false) ? new cmsRequest([], cmsRequest::CTX_INTERNAL) : $_request;

            $controller = cmsCore::getController($listener, $request);

            if ($controller->mb_installed && !$controller->isControllerInstalled($listener)) {
                unset($controller);
                continue;
            }

            cmsDebugging::pointStart('events');

            // >> @WebMan - debugx
            $dbg_hook_data = $data;
            // << @WebMan - debugx

            $data = $controller->runHook($event_name, [$data]);

            // >> @WebMan - debugx
            cmsDebugging::pointProcess('events', function () use($event_name, $listener, $data, $dbg_hook_data) {
                return [
                    'data'         => 'hook :: '.$listener.' => '.$event_name,
                    'x_name'       => $event_name,
                    'x_controller' => $listener,
                    'x_action'     => 'hook',
                    'x_data'       => $dbg_hook_data,
                    'x_result'     => $data
                ];
            }, 1);
            // << @WebMan - debugx

            unset($controller);
        }

        // >> @WebMan - debugx
        cmsDebugging::pointProcess('x_events', function () use($event_name, $dbg_event_data, $data) {
            return [
                'data'     => 'event :: '.$event_name,
                'x_name'   => $event_name,
                'x_action' => 'hook',
                'x_data'   => $dbg_event_data,
                'x_result' => $data
            ];
        });
        // << @WebMan - debugx

        return $data;
    }

    /**
     * Оповещает слушателей о произошедшем событии
     * Входящие данные $data передаются каждому слушателю в изначальном виде,
     * на выходе возвращается массив с ответами от каждого слушателя
     *
     * @param string $event_name Название события
     * @param mixed $data Параметр события
     * @param mixed $default_return Значение, возвращаемое по умолчанию если у события нет слушателей
     * @param object $_request Объект запроса
     * @return array Обработанный массив данных
     */
    public static function hookAll($event_name, $data = false, $default_return = null, $_request = false) {

        // >> @WebMan - debugx
        cmsDebugging::pointStart('x_events');
        // << @WebMan - debugx

        //получаем все активные контроллеры, привязанные к указанному событию
        $listeners = self::getEventListeners($event_name);

        //если активных контроллеров нет, возвращаем данные без изменений
        if (!$listeners) {

            // >> @WebMan - debugx
            cmsDebugging::pointProcess('events_empty', function () use($event_name, $data) {
                return [
                    'data'     => 'hookAll => '.$event_name,
                    'x_name'   => $event_name,
                    'x_action' => 'hookAll',
                    'x_data'   => $data
                ];
            }, 1);
            // << @WebMan - debugx

            return is_null($default_return) ? false : $default_return;
        }

        $results = [];

        //перебираем контроллеры и вызываем каждый из них, передавая $data
        foreach ($listeners as $listener) {

            $request = ($_request === false) ? new cmsRequest([], cmsRequest::CTX_INTERNAL) : $_request;

            $controller = cmsCore::getController($listener, $request);

            if ($controller->mb_installed && !$controller->isControllerInstalled($listener)) {
                unset($controller);
                continue;
            }

            cmsDebugging::pointStart('events');

            $result = $controller->runHook($event_name, [$data]);

            if ($result !== false) {
                $results[$listener] = $result;
            }

            // >> @WebMan - debugx
            cmsDebugging::pointProcess('events', function () use($event_name, $listener, $data, $result) {
                return [
                    'data'         => 'hookAll :: '.$listener.' => '.$event_name,
                    'x_name'       => $event_name,
                    'x_controller' => $listener,
                    'x_action'     => 'hookAll',
                    'x_data'       => $data,
                    'x_result'     => $result
                ];
            }, 1);
            // << @WebMan - debugx

            unset($controller);
        }

        // >> @WebMan - debugx
        cmsDebugging::pointProcess('x_events', function () use($event_name, $data, $results) {
            return [
                'data'     => 'event :: '.$event_name,
                'x_name'   => $event_name,
                'x_action' => 'hookAll',
                'x_data'   => $data,
                'x_result' => $results
            ];
        });
        // << @WebMan - debugx

        return $results;
    }

    /**
     * Возвращает список всех слушателей указанного события
     * @param string $event_name Название события
     * @return array Список слушателей
     */
    public static function getEventListeners($event_name) {

        $listeners = [];

        if (self::$structure === null) {
            self::$structure = self::getAllListeners();
        }

        if (isset(self::$structure[$event_name])) {
            $listeners = self::$structure[$event_name];
        }

        return $listeners;
    }

    /**
     * Возвращает список всех слушателей для всех событий
     * @return array
     */
    public static function getAllListeners() {

        $cache     = cmsCache::getInstance();
        $cache_key = 'events';

        if (false !== ($structure = $cache->get($cache_key))) {
            return $structure;
        }

        $manifests = cmsCore::getControllersManifests(cmsConfig::get('manifest_from_files'));
        if (!$manifests) {
            return [];
        }

        $structure = [];

        foreach ($manifests as $controller_name => $hooks) {

            if (!cmsController::enabled($controller_name)) {
                continue;
            }

            foreach ($hooks as $ordering => $event_name) {
                $structure[$event_name][$ordering] = $controller_name;
            }
        }

        foreach ($structure as $event_name => $controllers) {
            ksort($structure[$event_name]);
        }

        $cache->set($cache_key, $structure, 86400);

        return $structure;
    }

}
